//  AviationWeatherStation.m
//  Created on 10/28/15
//  Copyright © 2015 Ravenware Industries LLC, All rights reserved.

// What this is;
// This is an Objective-C object to encapsulate a aviation weather reporting station. It also contains
// some convinience methods for using this data.
// Why?
// One could carry around a textual list and parse it constantly,
// I thought this would be handier for things I was doing.  And I figured may as well post it.
// Where is this format defined?
// The format that defines the object is from the stations.txt file, which is created at the National
// Center for Atmospheric Reseach and used throughout the Weather community
// The master continuously updated file is here;
// http://www.rap.ucar.edu/weather/surface/stations.txt
// with another continously updated version at the Aviation Weather server here
// http://www.aviationweather.gov/static/adds/metars/stations.txt

// Distributed under the Fredware License below:
// You have the explicit right to copy, redistribute, and use any content on this site provided that you;
// -Acknowledge and agree that any copyright, trademark, service mark,
//  or other is retained by C.K. Haun and Ravenware Industries LLC
// - Include a reference to Ravenware.com as the origin of the content
// - Acknowledge that there is no representation that any advice or techniques presented in
// this source code are optimum or even correct
// - If you use some code in your own code you include the acknowledgment "Some portions of this
//   product created by C.K. Haun and Ravenware Industries LLC"
// - You never expect or demand bug fixes or updates (asking politely is OK)
//- And most importantly, you agree that if you don't like something here,
//  you don't complain, you just create a better one yourself.




#import "AviationWeatherStation.h"

@implementation AviationWeatherStation
@synthesize stateName;  // may not exist
@synthesize stateAbbreviation;
@synthesize stationName;
@synthesize ICAO;
@synthesize IATA;
@synthesize SYNOPIndex;
@synthesize latitude;
@synthesize longitude;
@synthesize ELEV;
@synthesize METAR;
@synthesize NEXRAD;
@synthesize aviationFlag;
@synthesize upperAirOrWind;
@synthesize observationType;
@synthesize officeType;
@synthesize plottingPriority;
@synthesize countryCode;
@synthesize refCon;  // optional




-(id)copyWithZone:(NSZone *)zone{
    AviationWeatherStation *returnValue=[[AviationWeatherStation alloc]init];
    if(self.stateName)
        returnValue.stateName=[self.stateName copyWithZone:zone];
    if(self.stateAbbreviation)
        returnValue.stateAbbreviation=[self.stateAbbreviation  copyWithZone:zone];
    if(self.stationName)
        returnValue. stationName=[self.stationName  copyWithZone:zone];
    if(self.ICAO)
        returnValue. ICAO=[self.ICAO   copyWithZone:zone];
    if(self.IATA)
        returnValue. IATA=[self.IATA  copyWithZone:zone];
    returnValue. SYNOPIndex=self.SYNOPIndex;
    returnValue. latitude=self.latitude ;
    returnValue. longitude=self.longitude;
    returnValue. ELEV=self.ELEV;
    returnValue. METAR=self.METAR;
    returnValue. NEXRAD=self.NEXRAD;
    returnValue. aviationFlag=self.aviationFlag;
    returnValue. upperAirOrWind=self.upperAirOrWind;
    returnValue. observationType=self.observationType;
    returnValue. officeType=self.officeType;
    returnValue. plottingPriority=self.plottingPriority;
    if(self.countryCode)
        returnValue. countryCode=[self.countryCode  copyWithZone:zone];
    if(self.refCon)
        returnValue.refCon = [self.refCon  copyWithZone:zone];
    
    return returnValue;
}

// Archiving
// Note that I am archiving chars as NSNumbers not as single bytes
// This is less efficient, but it provides a bit more robustness

- (void)encodeWithCoder:(NSCoder *)encoder
{
    [encoder encodeObject:self.stateName forKey:@"stateName"];
    [encoder encodeObject:self.stateAbbreviation forKey:@"stateAbbreviation"];
    [encoder encodeObject:self.stationName forKey:@"stationName"];
    [encoder encodeObject:self.ICAO forKey:@"ICAO"];
    [encoder encodeObject:self.IATA forKey:@"IATA"];
    [encoder encodeInteger:self.SYNOPIndex forKey:@"SYNOPIndex"];
    [encoder encodeDouble:self.latitude forKey:@"latitude"];
    [encoder encodeDouble:self.longitude forKey:@"longitude"];
    [encoder encodeInteger:self.ELEV forKey:@"ELEV"];
    [encoder encodeObject:[NSNumber numberWithChar:self.METAR] forKey:@"METAR"];
    [encoder encodeObject:[NSNumber numberWithChar:self.NEXRAD] forKey:@"NEXRAD"];
    [encoder encodeObject:[NSNumber numberWithChar:self.aviationFlag] forKey:@"aviationFlag"];
    [encoder encodeObject:[NSNumber numberWithChar:self.upperAirOrWind] forKey:@"upperAirOrWind"];
    [encoder encodeObject:[NSNumber numberWithChar:self.observationType] forKey:@"observationType"];
    [encoder encodeObject:[NSNumber numberWithChar:self.officeType] forKey:@"officeType"];
    [encoder encodeObject:[NSNumber numberWithChar:self.plottingPriority] forKey:@"plottingPriority"];
    [encoder encodeObject:self.countryCode forKey:@"countryCode"];
    [encoder encodeObject:self.refCon forKey:@"refCon"];

}

- (id)initWithCoder:(NSCoder *)decoder
{
    self = [super init];
    if (self) {
        self.stateName = [decoder decodeObjectForKey:@"stateName"];
        self.stateAbbreviation = [decoder decodeObjectForKey:@"stateAbbreviation"];
        self.stationName = [decoder decodeObjectForKey:@"stationName"];
        self.ICAO = [decoder decodeObjectForKey:@"ICAO"];
        self.IATA = [decoder decodeObjectForKey:@"IATA"];
        self.SYNOPIndex = [decoder decodeIntegerForKey:@"SYNOPIndex"];
        self.latitude = [decoder decodeDoubleForKey:@"latitude"];
        self.longitude = [decoder decodeDoubleForKey:@"longitude"];
        self.ELEV = [decoder decodeIntegerForKey:@"ELEV"];
        self.METAR=[[decoder decodeObjectForKey:@"METAR"] charValue];
        self.NEXRAD=[[decoder decodeObjectForKey:@"NEXRAD"] charValue];
        self.aviationFlag=[[decoder decodeObjectForKey:@"aviationFlag"] charValue];
        self.upperAirOrWind=[[decoder decodeObjectForKey:@"upperAirOrWind"] charValue];
        self.observationType=[[decoder decodeObjectForKey:@"observationType"] charValue];
        self.officeType=[[decoder decodeObjectForKey:@"officeType"] charValue];
        self.plottingPriority=[[decoder decodeObjectForKey:@"plottingPriority"] charValue];
        self.countryCode = [decoder decodeObjectForKey:@"countryCode"];
        self.refCon = [decoder decodeObjectForKey:@"refCon"];

    }
    return self;
}


#pragma mark Favorites
// This is a sample use of the refCon dictionary 
// Note that this is not a property because it isn't an ivar because it may not
// exist in certain implementations
-(BOOL) isFavorite{
    BOOL retVal=NO;
    if(self.refCon){
        NSNumber *valueAvailable = [refCon objectForKey:@"CKFavorite"];
        if(valueAvailable){
            retVal = [valueAvailable boolValue];
        }
    } else {
    }
    
    return retVal;
}

-(void) setFavorite:(BOOL) yesOrNo{
    if(self.refCon==nil)self.refCon = [[NSMutableDictionary alloc]init];
    NSNumber *addIT = [NSNumber numberWithBool:yesOrNo];
    [self.refCon setObject:addIT forKey:@"CKFavorite"   ];

    
}
-(void)toggleFavoriteStatus{
    BOOL iii = [self isFavorite];
    if(iii)iii=NO; else iii=YES;
    [self setFavorite:iii] ;
}
@end
