//
//  AviationWeatherStationCollection.m
//
//  Created by C.K. Haun on 11/9/15.
//  Copyright © 2015 Ravenware Industries LLC. All rights reserved.
// License
// You have the explicit right to copy, redistribute, and use any of this content provided that you;
// Acknowledge and agree that any copyright, trademark, service mark, or other is retained by C.K. Haun and Ravenware Industries LLC
// Include a reference to Ravenware.com as the origin of the content
// Acknowledge that there is no representation that any advice or techniques presented on this site are optimum or even correct
// If you use some content in your own content you include the acknowledgment "Some portions of this product created by C.K. Haun and Ravenware Industries LLC"
// You never expect or demand bug fixes or updates
// And most importantly, you agree that if you don't like something here, you don't complain, you just create a better one yourself.

// What this is;
// This is an Objective-C object to encapsulate a set of aviation weather reporting
// stations. It also contains some convinience methods for using this data.
// This is (obviously) ARC coded, no retain/releasing
//
// Why?
// One could carry around a textual list and parse it constantly,
// I thought this would be handier for things I was doing.
// And I figured may as well post it.
// Where is the station format defined?
// The format that defines the object is from the stations.txt file, which is created at the National
// Center for Atmospheric Reseach and used throughout the Weather community
// The master continuously updated file is here;
// http://www.rap.ucar.edu/weather/surface/stations.txt
// with another continously updated version at the Aviation Weather server here
// http://www.aviationweather.gov/static/adds/metars/stations.txt

// Distributed under the Fredware License below:
// You have the explicit right to copy, redistribute, and use any content on this site provided that you;
// -Acknowledge and agree that any copyright, trademark, service mark,
//  or other is retained by C.K. Haun and Ravenware Industries LLC
// - Include a reference to Ravenware.com as the origin of the content
// - Acknowledge that there is no representation that any advice or techniques presented in
// this source code are optimum or even correct
// - If you use some code in your own code you include the acknowledgment "Some portions of this
//   product created by C.K. Haun and Ravenware Industries LLC"
// - You never expect or demand bug fixes or updates (asking politely is OK)
//- And most importantly, you agree that if you don't like something here,
//  you don't complain, you just create a better one yourself.

#import "AviationWeatherStationCollection.h"

@implementation AviationWeatherStationCollection
@synthesize stations,lastUpdated,fromURL,numberOfStations,stateName,stateByStateCollection;

-(NSArray *)stationsWithTAFCapability{
    NSArray *returnValue;
    NSMutableArray *temp=[[NSMutableArray alloc]init];
    for(AviationWeatherStation *currentStation in stations){
        if (currentStation.aviationFlag =='T') {

            [temp addObject:currentStation];
        }
        
    }
    returnValue=[NSArray arrayWithArray:temp];
    return returnValue;
    
}

-(NSArray *)stationsWithMETARCapability{
    NSArray *returnValue;
    NSMutableArray *temp=[[NSMutableArray alloc]init];
    for(AviationWeatherStation *currentStation in stations){
        if (currentStation.METAR =='X') {
            
            [temp addObject:currentStation];
        }
        
    }
    returnValue=[NSArray arrayWithArray:temp];
    return returnValue;
    
}

-(AviationWeatherStation *)stationWithICAOID:(NSString *)thisID{
    AviationWeatherStation *returnValue;
    for(AviationWeatherStation *currentStation in stations){
        if([currentStation.ICAO isEqualToString:thisID])
        {
            returnValue=currentStation;
            break;
        }
    }
    return returnValue;
}
// cascade, first to have a string wins
// May return a 0 count array of course
-(NSArray *)stationArrayWithPartialICAOID:(NSString *)thisID orName:(NSString *)name orState:(NSString *)state{
    NSArray *returnValue;
    NSInteger searchingOn;
    NSMutableArray *working = [[NSMutableArray alloc]init];
    NSString *toCheck;
    NSString *onThis;
    if(thisID){
        searchingOn=0;
        onThis=thisID ;
    } else {
        if(name){
            searchingOn =1;
            onThis=name;
        } else {
            if(state){
                searchingOn=2;
                onThis=state;
            }else {
                searchingOn=3;
            }
        }
    }
    for(AviationWeatherStation *current in stations){
        if(searchingOn ==3)break;
        switch (searchingOn){
            case 0:
                toCheck=current.ICAO;
                break;
                
            case 1:
                toCheck=current.stationName;
                break;
            case 2:
                toCheck = current.stateName;
                break;
            case 3:
                
                break;
        }
        
        ///////////////
        toCheck=[toCheck lowercaseString];
        onThis = [onThis lowercaseString];
        //If self or substring have 0 length they cannot match
        //This can have odd results with NSRange
        if([toCheck     length] == 0 || [onThis length] == 0){
            //donothing
        } else {
            NSRange textRange;
            
            textRange = [toCheck rangeOfString:onThis];
            
            if(textRange.location != NSNotFound)
            {
                [working addObject:current];
            }else{
                
            }
        }
        //////////////
        
        
    }

    returnValue = [NSArray arrayWithArray:working];
    
    return returnValue;
}


// Create a station collection for each discrete state/region
// Probably a more efficient way to do this, but I don't care
-(void)createStateCollections:(BOOL)usOnly{
    // I'm going to spin this in a block since it may take a bit.
    AviationWeatherStationCollection *many[300];
    NSString *currentStationState;
    AviationWeatherStationCollection *addingHere;
    int qq;
    for(qq=0;qq<300;qq++)many[qq]=0;
    for(AviationWeatherStation *current in stations){
        // we have to go my the full state name here since some locales in stations.txt
        // do not have an abbreviation
        currentStationState = current.stateName;
        for (qq=0;qq<300;qq++) {
            addingHere = many[qq];

            if(addingHere && [currentStationState isEqualToString:addingHere.stateName]){
                [addingHere.stations addObject:current];
                addingHere.numberOfStations++;
                break;
            }
            if(addingHere == 0){
                if(addingHere == 0){
                    addingHere = [[AviationWeatherStationCollection alloc]init];
                    // qq should be set to the empty slot
                    addingHere.stateName=current.stateName;
                    addingHere.stations=[[NSMutableArray alloc]init];
                    many[qq]=addingHere;
                    [addingHere.stations addObject:current];
                    break;
                    
                }

                
                
                break;  // might create the station here
            }

            
        }
    
    
    }

    self.stateByStateCollection=[[NSMutableArray alloc]init];
    for (qq=0;qq<300; qq++) {
        addingHere=many[qq];
        if(addingHere){
            [self.stateByStateCollection addObject:addingHere];
        }else{
            break;
        }
    }

}
// distance is in miles
-(NSMutableArray *)stationsWithinDistanceFromStation:(AviationWeatherStation *)station within:(NSInteger)distance{
    NSMutableArray *retVal=[[NSMutableArray alloc]init];
    CLLocation *loc1 =[[CLLocation alloc]initWithLatitude:station.latitude longitude:station.longitude];
    
    NSInteger dist=0;
    NSInteger dist2=0;
    for(AviationWeatherStation *thisStation in self.stations){
        if(![thisStation isEqual:station ]){
        CLLocation *loc2=[[CLLocation alloc]initWithLatitude:thisStation.latitude longitude:thisStation.longitude];
        dist = [loc1 distanceFromLocation:loc2];
        // order results in different distances sometimes, so checking both ways.
        dist2 = [loc2 distanceFromLocation:loc1];
#if 0
        if(dist != dist2){
            //            NSLog (@"dist different");
        }
#endif
        dist = dist*0.000621371;  // meters to miles
        dist2 = dist2*0.000621371;

        if(dist <distance || dist2 <distance){
            [retVal addObject:thisStation];
        }
    }
    }
    
    return(retVal);
}

-(AviationWeatherStation *)stationClosestToLocation:(CLLocation *)location{AviationWeatherStation *retVal = nil;
    CLLocationDistance dist =0;
    CLLocationDistance dist2 =0;

    CLLocationDistance closestSoFar =40075/2;
    // Circumfernece is 40,000 meters (max), so 1/2 way around the world, can't be further away than that.
    for(AviationWeatherStation *thisStation in self.stations){
            CLLocation *loc2=[[CLLocation alloc]initWithLatitude:thisStation.latitude longitude:thisStation.longitude];
            dist = [location distanceFromLocation:loc2];
        dist2 = [loc2 distanceFromLocation:location];

        if(dist < closestSoFar || dist2 <closestSoFar){
            closestSoFar=dist;
            retVal=thisStation  ;
            // of coursee don't break, keep looking
        }
        }
    return (retVal);
    
}

-(NSMutableArray *)stationsClosestToLocation:(CLLocation *)location within:(NSUInteger) distance
{

    NSMutableArray *retVal=[[NSMutableArray alloc]init];
    CLLocation *loc1 =location;
    
    NSInteger dist=0;
    NSInteger dist2=0;
    for(AviationWeatherStation *thisStation in self.stations){
            CLLocation *loc2=[[CLLocation alloc]initWithLatitude:thisStation.latitude longitude:thisStation.longitude];
            dist = [loc1 distanceFromLocation:loc2];
            // order results in different distances sometimes, so checking both ways.
            dist2 = [loc2 distanceFromLocation:loc1];
#if 0
        // for debugging
        if(dist != dist2){
                //            NSLog (@"dist different");
            }
#endif
            dist = dist*0.000621371;  // meters to miles
            dist2 = dist2*0.000621371;
            
            if(dist <distance || dist2 <distance){
                [retVal addObject:thisStation];
            }
        
    }

    return (retVal);
    
}




-(id)copyWithZone:(NSZone *)zone{
    AviationWeatherStationCollection *returnValue =[[[self class] allocWithZone:zone] init];
    if(self.lastUpdated)
        returnValue.lastUpdated=[self.lastUpdated copyWithZone:zone];
    if(self.fromURL )
        returnValue.fromURL=[self.fromURL copyWithZone:zone];
    returnValue.numberOfStations=self.numberOfStations;
    if(self.stations){
        returnValue.stations=[[NSMutableArray alloc]initWithCapacity:returnValue.numberOfStations];
        for(AviationWeatherStation *currentStation in self.stations){
            [returnValue.stations addObject:[currentStation copyWithZone:zone]];
        }
    
    }
    if(self.stateByStateCollection){
        returnValue.stateByStateCollection = [self.stateByStateCollection copyWithZone:zone];
    }
    return (returnValue);
}

- (void)encodeWithCoder:(NSCoder *)encoder
{
    [encoder encodeObject:self.lastUpdated forKey:@"lastUpdated"];
    [encoder encodeObject:[NSNumber numberWithInteger:self.numberOfStations] forKey:@"numberOfStations"];

    [encoder encodeObject:self.fromURL forKey:@"fromURL"];
    [encoder encodeObject:self.stations forKey:@"stations"];
}

- (id)initWithCoder:(NSCoder *)decoder
{
    self = [super init];
    if (self) {
            self.lastUpdated = [decoder decodeObjectForKey:@"lastUpdated"];
        self.numberOfStations=  [[decoder decodeObjectForKey:@"numberOfStations"] integerValue];

        self.fromURL = [decoder decodeObjectForKey:@"fromURL"];
        self.stations = [decoder decodeObjectForKey:@"stations"];
    }
    return self;
}

@end
